/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_DECISION4SM_H
#define DPC_IMPL_TYPE_DECISION4SM_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint32.h"
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "impl_type_boolean.h"
#include "dpc/impl_type_vector_objecttrim.h"

namespace dpc {
struct Decision4SM {
    ::UInt32 ba_elk_tgt_id{ 0U };
    ::UInt8 ba_ldp_elk_md{ 0U };
    ::UInt8 ba_elk_tgt_dngr_lvl{ 0U };
    ::Float ba_lc_end_pnt_x{ 0.0F };
    ::Float ba_lc_end_pnt_y{ 0.0F };
    ::UInt32 ba_alc_abl_tgt_id{ 0U };
    ::UInt8 ba_alc_abl_tgt_dngr_lvl{ 0U };
    ::Float ba_dyn_allw_max_spd{ 0.0F };
    ::UInt32 ba_opt_corridor_id{ 0U };
    ::UInt32 ba_opt_target_obj_id{ 0U };
    ::UInt8 ba_opt_action{ 0U };
    ::UInt8 ba_takeover_req{ 0U };
    ::Boolean ba_lane_change_abolition_avlbl{ false };
    ::dpc::vector_ObjectTrim ba_obj_list2sm;
    ::UInt8 ba_drcn_ind_req{ 0U };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(ba_elk_tgt_id);
        fun(ba_ldp_elk_md);
        fun(ba_elk_tgt_dngr_lvl);
        fun(ba_lc_end_pnt_x);
        fun(ba_lc_end_pnt_y);
        fun(ba_alc_abl_tgt_id);
        fun(ba_alc_abl_tgt_dngr_lvl);
        fun(ba_dyn_allw_max_spd);
        fun(ba_opt_corridor_id);
        fun(ba_opt_target_obj_id);
        fun(ba_opt_action);
        fun(ba_takeover_req);
        fun(ba_lane_change_abolition_avlbl);
        fun(ba_obj_list2sm);
        fun(ba_drcn_ind_req);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(ba_elk_tgt_id);
        fun(ba_ldp_elk_md);
        fun(ba_elk_tgt_dngr_lvl);
        fun(ba_lc_end_pnt_x);
        fun(ba_lc_end_pnt_y);
        fun(ba_alc_abl_tgt_id);
        fun(ba_alc_abl_tgt_dngr_lvl);
        fun(ba_dyn_allw_max_spd);
        fun(ba_opt_corridor_id);
        fun(ba_opt_target_obj_id);
        fun(ba_opt_action);
        fun(ba_takeover_req);
        fun(ba_lane_change_abolition_avlbl);
        fun(ba_obj_list2sm);
        fun(ba_drcn_ind_req);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("ba_elk_tgt_id", ba_elk_tgt_id);
        fun("ba_ldp_elk_md", ba_ldp_elk_md);
        fun("ba_elk_tgt_dngr_lvl", ba_elk_tgt_dngr_lvl);
        fun("ba_lc_end_pnt_x", ba_lc_end_pnt_x);
        fun("ba_lc_end_pnt_y", ba_lc_end_pnt_y);
        fun("ba_alc_abl_tgt_id", ba_alc_abl_tgt_id);
        fun("ba_alc_abl_tgt_dngr_lvl", ba_alc_abl_tgt_dngr_lvl);
        fun("ba_dyn_allw_max_spd", ba_dyn_allw_max_spd);
        fun("ba_opt_corridor_id", ba_opt_corridor_id);
        fun("ba_opt_target_obj_id", ba_opt_target_obj_id);
        fun("ba_opt_action", ba_opt_action);
        fun("ba_takeover_req", ba_takeover_req);
        fun("ba_lane_change_abolition_avlbl", ba_lane_change_abolition_avlbl);
        fun("ba_obj_list2sm", ba_obj_list2sm);
        fun("ba_drcn_ind_req", ba_drcn_ind_req);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("ba_elk_tgt_id", ba_elk_tgt_id);
        fun("ba_ldp_elk_md", ba_ldp_elk_md);
        fun("ba_elk_tgt_dngr_lvl", ba_elk_tgt_dngr_lvl);
        fun("ba_lc_end_pnt_x", ba_lc_end_pnt_x);
        fun("ba_lc_end_pnt_y", ba_lc_end_pnt_y);
        fun("ba_alc_abl_tgt_id", ba_alc_abl_tgt_id);
        fun("ba_alc_abl_tgt_dngr_lvl", ba_alc_abl_tgt_dngr_lvl);
        fun("ba_dyn_allw_max_spd", ba_dyn_allw_max_spd);
        fun("ba_opt_corridor_id", ba_opt_corridor_id);
        fun("ba_opt_target_obj_id", ba_opt_target_obj_id);
        fun("ba_opt_action", ba_opt_action);
        fun("ba_takeover_req", ba_takeover_req);
        fun("ba_lane_change_abolition_avlbl", ba_lane_change_abolition_avlbl);
        fun("ba_obj_list2sm", ba_obj_list2sm);
        fun("ba_drcn_ind_req", ba_drcn_ind_req);
    }

    friend bool operator==(const ::dpc::Decision4SM& lhs, const ::dpc::Decision4SM& rhs) noexcept
    {
        return (lhs.ba_elk_tgt_id == rhs.ba_elk_tgt_id) && (lhs.ba_ldp_elk_md == rhs.ba_ldp_elk_md) && (lhs.ba_elk_tgt_dngr_lvl == rhs.ba_elk_tgt_dngr_lvl) && (fabs(static_cast<double>(lhs.ba_lc_end_pnt_x - rhs.ba_lc_end_pnt_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ba_lc_end_pnt_y - rhs.ba_lc_end_pnt_y)) < DBL_EPSILON) && (lhs.ba_alc_abl_tgt_id == rhs.ba_alc_abl_tgt_id) && (lhs.ba_alc_abl_tgt_dngr_lvl == rhs.ba_alc_abl_tgt_dngr_lvl) && (fabs(static_cast<double>(lhs.ba_dyn_allw_max_spd - rhs.ba_dyn_allw_max_spd)) < DBL_EPSILON) && (lhs.ba_opt_corridor_id == rhs.ba_opt_corridor_id) && (lhs.ba_opt_target_obj_id == rhs.ba_opt_target_obj_id) && (lhs.ba_opt_action == rhs.ba_opt_action) && (lhs.ba_takeover_req == rhs.ba_takeover_req) && (lhs.ba_lane_change_abolition_avlbl == rhs.ba_lane_change_abolition_avlbl) && (lhs.ba_obj_list2sm == rhs.ba_obj_list2sm) && (lhs.ba_drcn_ind_req == rhs.ba_drcn_ind_req);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_DECISION4SM_H
